﻿using PComm;
using PObject;
using PUtility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ClientForm
{
	delegate void LogHandler(PClient client, string log);
	delegate void FormHandler(Form form);

	public partial class MainForm : Form
	{
		private string serverIP = "127.0.0.1";
		private int serverPort = 37777;
		private int heartBeatPort = 37778;

		private PClient heartBeatClient;
		private PClient client;

		private LogHandler logHandler;
		private FormHandler formHandler;

		private System.Timers.Timer heartBeatTimer;
		private System.Timers.Timer GarbageTimer;

		public MainForm()
		{
			InitializeComponent();
			InitInstance();
		}

		private void InitInstance()
		{
			//AppSetting.Default.SetValue<string>("SERVER_IP", "127.0.0.105");
			serverIP = AppSetting.Default.GetValue<string>("SERVER_IP", "127.0.0.1");
			serverPort = AppSetting.Default.GetValue<int>("PORT", 37777);
			heartBeatPort = AppSetting.Default.GetValue<int>("HB_PORT", 37778);
		}

		private void ConnectToServer(string address)
		{
			IPAddress ipAddress = PUtil.ValidateAddress(address);
			if (ipAddress == null)
				return;

			string clientId = PUtil.GetRandomString(5);
			string heartBeatClientId = clientId + "_hb";

			logHandler = new LogHandler(WriteLog);
			formHandler = new FormHandler(ShowForm);

			heartBeatClient = new PClient(ipAddress, heartBeatPort, heartBeatClientId);
			heartBeatClient.OnConnected += CommonClient_OnConnected;
			heartBeatClient.OnSend += HeartBeatClient_OnSend;
			heartBeatClient.OnReceived += HeartBeatClient_OnReceived;
			heartBeatClient.OnErrorCatched += CommonClient_OnErrorCatched;
			heartBeatClient.OnDisconnected += CommonClient_OnDisconnected;

			client = new PClient(ipAddress, serverPort, clientId);
			client.OnConnected += CommonClient_OnConnected;
			client.OnSend += Client_OnSend;
			client.OnReceived += Client_OnReceived;
			client.OnErrorCatched += CommonClient_OnErrorCatched;
			client.OnDisconnected += CommonClient_OnDisconnected;

			heartBeatClient.Connect();
			UpdateHeartBeat();
			client.Connect();

			heartBeatTimer = new System.Timers.Timer();
			heartBeatTimer.Interval = 1000 * 10;
			heartBeatTimer.Elapsed += HeartBeatTimer_Elapsed;
			heartBeatTimer.Start();

			GarbageTimer = new System.Timers.Timer();
			GarbageTimer.Interval = 1000 * 10;
			GarbageTimer.Elapsed += GarbageTimer_Elapsed;
			GarbageTimer.Start();
		}

		private void DisconnectFromServer()
		{
			heartBeatClient.Close();
			heartBeatClient.Dispose();
			client.Close();
			client.Dispose();

			heartBeatTimer.Stop();
			GarbageTimer.Stop();
		}

		private void Client_OnSend(PClient sender, PDataType dataType, byte[] data)
		{
			string log = $"Send data: target: {sender.EndPoint.ToString()}, type: {dataType.ToString()}, size: {data.Length.ToString("#,###")}bytes";
			logHandler(sender, log);
		}

		private void CommonClient_OnDisconnected(PClient sender)
		{
			string log = $"Disconnected: {sender.EndPoint.ToString()}";
			logHandler(sender, log);
		}

		private void CommonClient_OnErrorCatched(PClient sender, string msg)
		{
			string log = $"Error: {msg}";
			logHandler(sender, log);
		}

		private void HeartBeatClient_OnSend(PClient sender, PDataType dataType, byte[] data)
		{
			//throw new NotImplementedException();
		}

		private void CommonClient_OnConnected(PClient sender)
		{
			string log = $"Connected: {sender.EndPoint.ToString()}";
			logHandler(sender, log);
		}

		private void ShowForm(Form form)
		{
			if (this.InvokeRequired)
			{
				this.Invoke((MethodInvoker)delegate () { ShowForm(form); });
			}
			else
			{
				form.Show();
			}
		}

		private void WriteLog(PClient client,string log)
		{
			if (this.InvokeRequired)
			{
				this.Invoke((MethodInvoker)delegate () { WriteLog(client, log); });
			}
			else
			{
				logConsole.AppendText($"[{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}] {client.ID}: {log}" + Environment.NewLine);
				//PUtil.ScrollToEnd(logConsole);
			}
		}

		private void HeartBeatTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			heartBeatClient.Send(PServer.HB_CHECK);
		}

		private void HeartBeatClient_OnReceived(PClient sender, PDataType dataType, byte[] data)
		{
			if (dataType != PDataType.SimpleString)
			{
				WriteLog(sender, "ERROR!!! WRONG DATA TYPE FOR HEART BEAT!!!");
				return;
			}

			string msg = Encoding.UTF8.GetString(data, 0, data.Length);
			if (msg != PServer.HB_CHECK)
			{
				WriteLog(sender, "WARNING!!! BAD HEARTBEAT!!!");
				client.Connect();
				heartBeatClient.Connect();
			}

			UpdateHeartBeat();
		}

		private void UpdateHeartBeat()
		{
			heartBeatStatus.Text = DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss");
		}

		private void Client_OnReceived(PClient sender, PDataType dataType, byte[] data)
		{
			throw new NotImplementedException();
		}

		private void GarbageTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			GC.Collect();
		}

		private void stringButton_Click(object sender, EventArgs e)
		{
			client.Send("String send test!");
		}

		private void objectButton_Click(object sender, EventArgs e)
		{
			Random rand = new Random();
			Person person = new Person()
			{
				Name = PUtil.GetRandomString(5),
				Age = rand.Next(20, 100)
			};

			byte[] data = PSerializer.Serialize(person);
			client.Send(PDataType.PersonObject, data);
		}

		private void imageButton_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Filter = "JPG File|*.jpg|PNG File|*.png|All File|*.*";
			if (dlg.ShowDialog() == DialogResult.Cancel)
				return;

			Image img = Image.FromFile(dlg.FileName);
			byte[] data = PUtil.ImageToBytes(img);
			client.Send(PDataType.ShowImage, data);

			GC.Collect();
		}

		private void fileButton_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Filter = "All File|*.*";
			if (dlg.ShowDialog() == DialogResult.Cancel)
				return;

			string filePath = dlg.FileName;
			string fileName = dlg.SafeFileName;

			PFileData fileData = new PFileData()
			{
				FileName = fileName,
				Data = PUtil.FileToBytes(filePath)
			};

			if (fileData.Data.Length >= 1000000000)
			{
				MessageBox.Show("1GB를 초과하는 데이터는 전송할 수 없습니다.");

				GC.Collect();
				return;
			}


			byte[] data = PSerializer.Serialize(fileData);
			client.Send(PDataType.SaveFile, data);

			GC.Collect();
		}

		private void startButton_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(serverIP))
			{
				MessageBox.Show("Empty address");
				return;
			}

			ConnectToServer(serverIP);
			startButton.Enabled = false;
		}
	}
}
