﻿using PComm;
using PObject;
using PUtility;
using ServerForm.ChildForm;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ServerForm
{
	delegate void LogHandler(PClient client, string log);
	delegate void FormHandler(Form form);

	public partial class MainForm : Form
	{
		private int serverPort = 37777;
		private int heartBeatPort = 37778;

		private PServer mainServer;
		private PServer heartBeatServer;

		private LogHandler logHandler;
		private FormHandler formHandler;

		private System.Timers.Timer clientCheckTimer;
		private System.Timers.Timer GarbageTimer;

		public MainForm()
		{
			InitializeComponent();
			InitInstance();
		}

		private void InitInstance()
		{
			serverPort = AppSetting.Default.GetValue<int>("PORT", 37777);
			heartBeatPort = AppSetting.Default.GetValue<int>("HB_PORT", 37778);

			heartBeatServer = new PServer(heartBeatPort);
			mainServer = new PServer(serverPort);

			logHandler = new LogHandler(WriteLog);
			formHandler = new FormHandler(ShowForm);

			heartBeatServer.OnDataReceived += HeartBeatServer_OnDataReceived;
			mainServer.OnDataReceived += MainServer_OnDataReceived;

			heartBeatServer.Start();
			mainServer.Start();

			clientCheckTimer = new System.Timers.Timer();
			clientCheckTimer.Interval = 1000;
			clientCheckTimer.Elapsed += ClientCheckTimer_Elapsed;
			clientCheckTimer.Start();

			GarbageTimer = new System.Timers.Timer();
			GarbageTimer.Interval = 1000 * 10;
			GarbageTimer.Elapsed += GarbageTimer_Elapsed;
			GarbageTimer.Start();
		}

		private void WriteLog(PClient client, string log)
		{
			if (this.InvokeRequired)
			{
				this.Invoke((MethodInvoker)delegate () { WriteLog(client, log); });
			}
			else
			{
				logConsole.AppendText($"[{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}] {client.ID}: {log}" + Environment.NewLine);
				//PUtil.ScrollToEnd(logConsole);
			}
		}

		private void ShowForm(Form form)
		{
			if (this.InvokeRequired)
			{
				this.Invoke((MethodInvoker)delegate () { ShowForm(form); });
			}
			else
			{
				form.Show();
			}
		}

		private void CheckClientInfo(List<PClientInfo> clinetInfoList)
		{
			if (this.InvokeRequired)
			{
				this.Invoke((MethodInvoker)delegate () { CheckClientInfo(clinetInfoList); });
			}
			else
			{
				if (clientListBox.Items != null && clientListBox.Items.Count > 1)
					clientListBox.Items.Clear();

				foreach (PClientInfo clientInfo in clinetInfoList)
				{
					PClient client = clientInfo.Client;
					string clientView = $"{client.ID}({client.EndPoint.Address}:{client.EndPoint.Port})";
					clientListBox.Items.Add(clientView);
				}
			}
		}

		private void MainServer_OnDataReceived(PClientInfo sender, PDataType dataType, byte[] data)
		{
			DataManager.Instance.Process(sender, dataType, data, logHandler, formHandler);
		}


		private void HeartBeatServer_OnDataReceived(PClientInfo sender, PDataType dataType, byte[] data)
		{
			if (dataType != PDataType.SimpleString)
			{
				logHandler(sender.Client, "ERROR!!! WRONG DATA TYPE FOR HEART BEAT!!!");
				return;
			}

			string msg = Encoding.UTF8.GetString(data, 0, data.Length);
			if (msg != PServer.HB_CHECK)
				logHandler(sender.Client, "WARNING!!! BAD HEARTBEAT!!!");

			sender.Client.Send(PServer.HB_CHECK);
			UpdateHeartBeat();
		}

		private void UpdateHeartBeat()
		{
			heartBeatStatus.Text = DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss");
		}

		private void ClientCheckTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			List<PClientInfo> clientInfoList = new List<PClientInfo>(mainServer.ClientInfoList.Count + heartBeatServer.ClientInfoList.Count);
			clientInfoList.AddRange(mainServer.ClientInfoList);
			clientInfoList.AddRange(heartBeatServer.ClientInfoList);

			CheckClientInfo(clientInfoList);
		}

		private void GarbageTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			GC.Collect();
		}
	}
}
