﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Timers;

namespace Server_Listener.Communication
{
	delegate void SocketAcceptedHandler(object sender, SocketAcceptedEventArgs e);

	internal class PListener
	{
		public Socket BaseSocket { get; private set; }

		public bool IsRunning { get; private set; } = false;

		public int Port { get; private set; } = -1;

		public event SocketAcceptedHandler SocketAccepted;

		public PListener(int port)
		{
			this.Port = port;
		}

		public void Start()
		{
			if (this.IsRunning)
				return;

			this.IsRunning = true;
			this.BaseSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
			this.BaseSocket.Bind(new IPEndPoint(IPAddress.Any, this.Port));
			this.BaseSocket.Listen(100);
			this.BaseSocket.BeginAccept(AcceptCallback, null);
		}

		public void Stop()
		{
			if (!IsRunning)
				return;

			this.IsRunning = false;
			this.BaseSocket.Close();
		}

		private void AcceptCallback(IAsyncResult ar)
		{
			try
			{
				Socket socket = this.BaseSocket.EndAccept(ar);
				if (SocketAccepted != null)
					SocketAccepted(this, new SocketAcceptedEventArgs(socket));

				if (IsRunning)
					this.BaseSocket.BeginAccept(AcceptCallback, null);
			}
			catch (Exception ex)
			{
				Debug.WriteLine($"[ERROR] ServerAcceptCallback: {ex.Message}");
			}
		}
	}
}
