﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Timers;

namespace Server_Listener.Communication
{
	internal class PServer
	{
		public delegate void ClientDataReceivedHandler(PClientInfo sender, byte[] data);
		public event ClientDataReceivedHandler OnDataReceived;

		private PListener listener;
		private Dictionary<string, PClientInfo> clientInfoDict;

		private Timer garbageTimer;

		private object garbageLock = new object();

		public PServer(int port)
		{
			listener = new PListener(port);
			clientInfoDict = new Dictionary<string, PClientInfo>();

			listener.SocketAccepted += Listener_SocketAccepted;

			garbageTimer = new Timer();
			garbageTimer.Interval = 10 * 1000;
			garbageTimer.Elapsed += GarbageTimer_Elapsed;
		}

		public void Start()
		{
			listener.Start();
			garbageTimer.Start();
		}

		private void Listener_SocketAccepted(object sender, SocketAcceptedEventArgs e)
		{
			PClient client = new PClient(e.Accepted);
			client.Received += Client_Received;
			client.Disconnected += Client_Disconnected;

			Debug.WriteLine($"Client accepted: {client.ID} @ { DateTime.Now}");
		}

		private void Client_Disconnected(PClient sender)
		{
			if (clientInfoDict.ContainsKey(sender.ID))
			{
				lock (garbageLock)
				{
					clientInfoDict.Remove(sender.ID);
				}
			}
		}

		private void Client_Received(PClient sender, byte[] data)
		{
			lock (garbageLock)
			{
				if (!clientInfoDict.ContainsKey(sender.ID))
					clientInfoDict.Add(sender.ID, new PClientInfo());

				DateTime now = DateTime.Now;

				clientInfoDict[sender.ID].Client = sender;
				clientInfoDict[sender.ID].CreatedAt = now;
				clientInfoDict[sender.ID].LastReceivedAt = now;
			}

			string msg = Encoding.UTF8.GetString(data);
			clientInfoDict[sender.ID].LastMessage = msg;

			if (OnDataReceived != null && clientInfoDict.ContainsKey(sender.ID))
				OnDataReceived(clientInfoDict[sender.ID], data);

			Debug.WriteLine(($"Client message({sender.ID}): {msg} @ { DateTime.Now}"));
		}

		private void GarbageTimer_Elapsed(object sender, ElapsedEventArgs e)
		{
			List<string> garbageKeys = new List<string>();
			foreach (KeyValuePair<string, PClientInfo> item in clientInfoDict)
			{
				if (!item.Value.Client.Connected)
					garbageKeys.Add(item.Key);
			}

			foreach (string key in garbageKeys)
			{
				clientInfoDict.Remove(key);
			}
		}
	}
}
